@@ -18,6 +18,10 @@
  */
 
 #include <src/Channel.h>
+#include <gee/iterable.h>
+#include <gee/iterator.h>
+#include <gee/collection.h>
+#include <gee/arraylist.h>
 #include "src/Schedule.h"
 
 
