@@ -22,7 +22,7 @@
 
 #include <glib.h>
 #include <glib-object.h>
-#include <gee.h>
+#include <gee/list.h>
 #include "src/DeviceGroup.h"
 #include "src/Device.h"
 
