@@ -18,6 +18,9 @@
  */
 
 #include <src/DeviceGroup.h>
+#include <gee/set.h>
+#include <gee/collection.h>
+#include <gee/hashset.h>
 #include "src/EPGScanner.h"
 #include "src/Recorder.h"
 
