@@ -22,8 +22,9 @@
 
 #include <glib.h>
 #include <glib-object.h>
-#include <gee.h>
+#include <gee/iterable.h>
 #include <gio/gio.h>
+#include <gee/iterator.h>
 #include <stdlib.h>
 #include <string.h>
 #include "src/Device.h"
