@@ -21,7 +21,10 @@
 #include <src/EPGScanner.h>
 #include <stdlib.h>
 #include <string.h>
-#include <gee.h>
+#include <gee/hashset.h>
+#include <gee/iterable.h>
+#include <gee/iterator.h>
+#include <gee/collection.h>
 #include "src/Factory.h"
 #include "src/Settings.h"
 #include "src/Channel.h"
