@@ -1,22 +1,43 @@
+/*
+ * Copyright (C) 2008,2009 Sebastian Pölsterl
+ *
+ * This file is part of GNOME DVB Daemon.
+ *
+ * GNOME DVB Daemon is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
+ *
+ * GNOME DVB Daemon is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with GNOME DVB Daemon.  If not, see <http://www.gnu.org/licenses/>.
+ */
 
 #ifndef __SRC_RECORDER_H__
 #define __SRC_RECORDER_H__
 
 #include <glib.h>
 #include <glib-object.h>
+#include <gee/iterable.h>
 #include <gee/map.h>
 #include <gee/set.h>
+#include <gee/iterator.h>
 #include <gio/gio.h>
 #include <stdlib.h>
 #include <string.h>
 #include "src/IDBusRecorder.h"
+#include "src/OverlapType.h"
 #include "src/DeviceGroup.h"
 #include "src/Timer.h"
 #include "src/Device.h"
-#include "src/EPGScanner.h"
 #include "src/Recording.h"
 #include <dbus/dbus.h>
 #include <dbus/dbus-glib.h>
+#include <dbus/dbus-glib-lowlevel.h>
 
 G_BEGIN_DECLS
 
@@ -28,8 +49,6 @@ G_BEGIN_DECLS
 #define DVB_IS_RECORDING_THREAD_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), DVB_TYPE_RECORDING_THREAD))
 #define DVB_RECORDING_THREAD_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), DVB_TYPE_RECORDING_THREAD, DVBRecordingThreadClass))
 
-typedef struct _DVBRecordingThread DVBRecordingThread;
-typedef struct _DVBRecordingThreadClass DVBRecordingThreadClass;
 typedef struct _DVBRecordingThreadPrivate DVBRecordingThreadPrivate;
 
 #define DVB_TYPE_RECORDER (dvb_recorder_get_type ())
@@ -39,8 +58,6 @@ typedef struct _DVBRecordingThreadPrivate DVBRecordingThreadPrivate;
 #define DVB_IS_RECORDER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), DVB_TYPE_RECORDER))
 #define DVB_RECORDER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), DVB_TYPE_RECORDER, DVBRecorderClass))
 
-typedef struct _DVBRecorder DVBRecorder;
-typedef struct _DVBRecorderClass DVBRecorderClass;
 typedef struct _DVBRecorderPrivate DVBRecorderPrivate;
 
 /**
@@ -84,6 +101,8 @@ GType dvb_recording_thread_get_type (void);
 DVBRecorder* dvb_recorder_construct (GType object_type, DVBDeviceGroup* dev);
 DVBRecorder* dvb_recorder_new (DVBDeviceGroup* dev);
 guint32 dvb_recorder_add_timer (DVBRecorder* self, DVBTimer* new_timer);
+void dvb_recorder_stop (DVBRecorder* self);
+DVBTimer* dvb_recorder_create_timer (DVBRecorder* self, guint channel, gint start_year, gint start_month, gint start_day, gint start_hour, gint start_minute, guint duration);
 void dvb_recorder_start_recording (DVBRecorder* self, DVBTimer* timer);
 void dvb_recorder_stop_recording (DVBRecorder* self, DVBTimer* timer);
 DVBDeviceGroup* dvb_recorder_get_DeviceGroup (DVBRecorder* self);
