@@ -1,3 +1,21 @@
+/*
+ * Copyright (C) 2008,2009 Sebastian Pölsterl
+ *
+ * This file is part of GNOME DVB Daemon.
+ *
+ * GNOME DVB Daemon is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
+ *
+ * GNOME DVB Daemon is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with GNOME DVB Daemon.  If not, see <http://www.gnu.org/licenses/>.
+ */
 
 #include <src/Scanner.h>
 #include <gee/arraylist.h>
@@ -12,9 +30,9 @@
 
 
 
-static char* string_replace (const char* self, const char* old, const char* replacement);
 static char* string_substring (const char* self, glong offset, glong len);
 static char* bool_to_string (gboolean self);
+static char* string_replace (const char* self, const char* old, const char* replacement);
 struct _DVBScannerPrivate {
 	DVBDevice* _Device;
 	GeeArrayList* new_channels;
@@ -33,6 +51,7 @@ enum  {
 	DVB_SCANNER_CHANNELS
 };
 #define DVB_SCANNER_BASE_PIDS "16:17:18"
+#define DVB_SCANNER_PIPELINE_TEMPLATE "dvbsrc name=dvbsrc adapter=%u frontend=%u pids=%s stats-reporting-interval=0 ! mpegtsparse ! fakesink silent=true"
 static void dvb_scanner_real_prepare (DVBScanner* self);
 static DVBScannedItem* dvb_scanner_real_get_scanned_item (DVBScanner* self, const GstStructure* structure);
 static DVBChannel* dvb_scanner_real_get_new_channel (DVBScanner* self);
@@ -52,58 +71,6 @@ static int _vala_strcmp0 (const char * str1, const char * str2);
 static void g_cclosure_user_marshal_VOID__UINT_UINT (GClosure * closure, GValue * return_value, guint n_param_values, const GValue * param_values, gpointer invocation_hint, gpointer marshal_data);
 static void g_cclosure_user_marshal_VOID__UINT_UINT_STRING_STRING_STRING_BOOLEAN (GClosure * closure, GValue * return_value, guint n_param_values, const GValue * param_values, gpointer invocation_hint, gpointer marshal_data);
 
-static char* string_replace (const char* self, const char* old, const char* replacement) {
-	GError * inner_error;
-	g_return_val_if_fail (self != NULL, NULL);
-	g_return_val_if_fail (old != NULL, NULL);
-	g_return_val_if_fail (replacement != NULL, NULL);
-	inner_error = NULL;
-	{
-		char* _tmp0;
-		GRegex* _tmp1;
-		GRegex* regex;
-		char* _tmp2;
-		char* _tmp3;
-		_tmp0 = NULL;
-		_tmp1 = NULL;
-		regex = (_tmp1 = g_regex_new (_tmp0 = g_regex_escape_string (old, -1), 0, 0, &inner_error), _tmp0 = (g_free (_tmp0), NULL), _tmp1);
-		if (inner_error != NULL) {
-			if (inner_error->domain == G_REGEX_ERROR) {
-				goto __catch20_g_regex_error;
-			}
-			goto __finally20;
-		}
-		_tmp2 = g_regex_replace_literal (regex, self, (glong) (-1), 0, replacement, 0, &inner_error);
-		if (inner_error != NULL) {
-			(regex == NULL) ? NULL : (regex = (g_regex_unref (regex), NULL));
-			if (inner_error->domain == G_REGEX_ERROR) {
-				goto __catch20_g_regex_error;
-			}
-			goto __finally20;
-		}
-		_tmp3 = NULL;
-		return (_tmp3 = _tmp2, (regex == NULL) ? NULL : (regex = (g_regex_unref (regex), NULL)), _tmp3);
-	}
-	goto __finally20;
-	__catch20_g_regex_error:
-	{
-		GError * e;
-		e = inner_error;
-		inner_error = NULL;
-		{
-			g_assert_not_reached ();
-			(e == NULL) ? NULL : (e = (g_error_free (e), NULL));
-		}
-	}
-	__finally20:
-	if (inner_error != NULL) {
-		g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, inner_error->message);
-		g_clear_error (&inner_error);
-		return NULL;
-	}
-}
-
-
 static char* string_substring (const char* self, glong offset, glong len) {
 	glong string_length;
 	const char* start;
@@ -133,6 +100,58 @@ static char* bool_to_string (gboolean self) {
 }
 
 
+static char* string_replace (const char* self, const char* old, const char* replacement) {
+	GError * inner_error;
+	g_return_val_if_fail (self != NULL, NULL);
+	g_return_val_if_fail (old != NULL, NULL);
+	g_return_val_if_fail (replacement != NULL, NULL);
+	inner_error = NULL;
+	{
+		char* _tmp0;
+		GRegex* _tmp1;
+		GRegex* regex;
+		char* _tmp2;
+		char* _tmp3;
+		_tmp0 = NULL;
+		_tmp1 = NULL;
+		regex = (_tmp1 = g_regex_new (_tmp0 = g_regex_escape_string (old, -1), 0, 0, &inner_error), _tmp0 = (g_free (_tmp0), NULL), _tmp1);
+		if (inner_error != NULL) {
+			if (inner_error->domain == G_REGEX_ERROR) {
+				goto __catch22_g_regex_error;
+			}
+			goto __finally22;
+		}
+		_tmp2 = g_regex_replace_literal (regex, self, (glong) (-1), 0, replacement, 0, &inner_error);
+		if (inner_error != NULL) {
+			(regex == NULL) ? NULL : (regex = (g_regex_unref (regex), NULL));
+			if (inner_error->domain == G_REGEX_ERROR) {
+				goto __catch22_g_regex_error;
+			}
+			goto __finally22;
+		}
+		_tmp3 = NULL;
+		return (_tmp3 = _tmp2, (regex == NULL) ? NULL : (regex = (g_regex_unref (regex), NULL)), _tmp3);
+	}
+	goto __finally22;
+	__catch22_g_regex_error:
+	{
+		GError * e;
+		e = inner_error;
+		inner_error = NULL;
+		{
+			g_assert_not_reached ();
+			(e == NULL) ? NULL : (e = (g_error_free (e), NULL));
+		}
+	}
+	__finally22:
+	if (inner_error != NULL) {
+		g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, inner_error->message);
+		g_clear_error (&inner_error);
+		return NULL;
+	}
+}
+
+
 static void dvb_scanner_real_prepare (DVBScanner* self) {
 	g_return_if_fail (self != NULL);
 	g_critical ("Type `%s' does not implement abstract method `dvb_scanner_prepare'", g_type_name (G_TYPE_FROM_INSTANCE (self)));
@@ -211,39 +230,33 @@ void dvb_scanner_Run (DVBScanner* self) {
 	_tmp0 = NULL;
 	self->channels = (_tmp0 = dvb_channel_list_new (NULL), (self->channels == NULL) ? NULL : (self->channels = (g_object_unref (self->channels), NULL)), _tmp0);
 	{
-		char* _tmp4;
-		char* _tmp3;
-		char* _tmp2;
 		char* _tmp1;
-		GstElement* _tmp5;
-		GstElement* _tmp6;
-		GstElement* _tmp7;
-		_tmp4 = NULL;
-		_tmp3 = NULL;
-		_tmp2 = NULL;
+		GstElement* _tmp2;
+		GstElement* _tmp3;
+		GstElement* _tmp4;
 		_tmp1 = NULL;
-		_tmp5 = NULL;
-		_tmp6 = (_tmp5 = gst_parse_launch (_tmp4 = g_strconcat (_tmp3 = g_strconcat (_tmp1 = g_strdup_printf ("dvbsrc name=dvbsrc adapter=%u frontend=%u ", dvb_device_get_Adapter (self->priv->_Device), dvb_device_get_Frontend (self->priv->_Device)), _tmp2 = g_strdup_printf ("pids=%s stats-reporting-interval=0 ", DVB_SCANNER_BASE_PIDS), NULL), "! mpegtsparse ! fakesink silent=true", NULL), &inner_error), _tmp4 = (g_free (_tmp4), NULL), _tmp3 = (g_free (_tmp3), NULL), _tmp2 = (g_free (_tmp2), NULL), _tmp1 = (g_free (_tmp1), NULL), _tmp5);
+		_tmp2 = NULL;
+		_tmp3 = (_tmp2 = gst_parse_launch (_tmp1 = g_strdup_printf (DVB_SCANNER_PIPELINE_TEMPLATE, dvb_device_get_Adapter (self->priv->_Device), dvb_device_get_Frontend (self->priv->_Device), DVB_SCANNER_BASE_PIDS), &inner_error), _tmp1 = (g_free (_tmp1), NULL), _tmp2);
 		if (inner_error != NULL) {
-			goto __catch21_g_error;
-			goto __finally21;
+			goto __catch23_g_error;
+			goto __finally23;
 		}
-		_tmp7 = NULL;
-		self->pipeline = (_tmp7 = _tmp6, (self->pipeline == NULL) ? NULL : (self->pipeline = (gst_object_unref (self->pipeline), NULL)), _tmp7);
+		_tmp4 = NULL;
+		self->pipeline = (_tmp4 = _tmp3, (self->pipeline == NULL) ? NULL : (self->pipeline = (gst_object_unref (self->pipeline), NULL)), _tmp4);
 	}
-	goto __finally21;
-	__catch21_g_error:
+	goto __finally23;
+	__catch23_g_error:
 	{
 		GError * e;
 		e = inner_error;
 		inner_error = NULL;
 		{
-			g_error ("Scanner.vala:128: Could not create pipeline: %s", e->message);
+			g_error ("Scanner.vala:146: Could not create pipeline: %s", e->message);
 			(e == NULL) ? NULL : (e = (g_error_free (e), NULL));
 			return;
 		}
 	}
-	__finally21:
+	__finally23:
 	if (inner_error != NULL) {
 		g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, inner_error->message);
 		g_clear_error (&inner_error);
@@ -295,9 +308,9 @@ gboolean dvb_scanner_WriteChannelsToFile (DVBScanner* self, const char* path) {
 		writer = (_tmp1 = dvb_channel_list_writer_new (_tmp0 = g_file_new_for_path (path), &inner_error), (_tmp0 == NULL) ? NULL : (_tmp0 = (g_object_unref (_tmp0), NULL)), _tmp1);
 		if (inner_error != NULL) {
 			if (inner_error->domain == G_IO_ERROR) {
-				goto __catch22_g_io_error;
+				goto __catch24_g_io_error;
 			}
-			goto __finally22;
+			goto __finally24;
 		}
 		{
 			GeeIterator* _c_it;
@@ -311,9 +324,9 @@ gboolean dvb_scanner_WriteChannelsToFile (DVBScanner* self, const char* path) {
 					(_c_it == NULL) ? NULL : (_c_it = (g_object_unref (_c_it), NULL));
 					(writer == NULL) ? NULL : (writer = (g_object_unref (writer), NULL));
 					if (inner_error->domain == G_IO_ERROR) {
-						goto __catch22_g_io_error;
+						goto __catch24_g_io_error;
 					}
-					goto __finally22;
+					goto __finally24;
 				}
 				(c == NULL) ? NULL : (c = (g_object_unref (c), NULL));
 			}
@@ -323,25 +336,25 @@ gboolean dvb_scanner_WriteChannelsToFile (DVBScanner* self, const char* path) {
 		if (inner_error != NULL) {
 			(writer == NULL) ? NULL : (writer = (g_object_unref (writer), NULL));
 			if (inner_error->domain == G_IO_ERROR) {
-				goto __catch22_g_io_error;
+				goto __catch24_g_io_error;
 			}
-			goto __finally22;
+			goto __finally24;
 		}
 		ret = TRUE;
 		(writer == NULL) ? NULL : (writer = (g_object_unref (writer), NULL));
 	}
-	goto __finally22;
-	__catch22_g_io_error:
+	goto __finally24;
+	__catch24_g_io_error:
 	{
 		GError * e;
 		e = inner_error;
 		inner_error = NULL;
 		{
-			g_critical ("Scanner.vala:168: Could not write channels file: %s", e->message);
+			g_critical ("Scanner.vala:186: Could not write channels file: %s", e->message);
 			(e == NULL) ? NULL : (e = (g_error_free (e), NULL));
 		}
 	}
-	__finally22:
+	__finally24:
 	if (inner_error != NULL) {
 		g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, inner_error->message);
 		g_clear_error (&inner_error);
@@ -399,10 +412,10 @@ void dvb_scanner_add_structure_to_scan (DVBScanner* self, GstStructure* structur
 	}
 	item = dvb_scanner_get_scanned_item (self, structure);
 	if (!gee_collection_contains ((GeeCollection*) self->scanned_frequencies, item)) {
-		GstStructure* _tmp0;
-		g_debug ("Scanner.vala:205: Queueing new frequency %u", dvb_scanned_item_get_Frequency (item));
+		const GstStructure* _tmp0;
+		g_debug ("Scanner.vala:223: Queueing new frequency %u", dvb_scanned_item_get_Frequency (item));
 		_tmp0 = NULL;
-		g_queue_push_tail (self->frequencies, (_tmp0 = structure, structure = NULL, _tmp0));
+		g_queue_push_tail (self->frequencies, (_tmp0 = structure, (_tmp0 == NULL) ? NULL : gst_structure_copy (_tmp0)));
 		gee_collection_add ((GeeCollection*) self->scanned_frequencies, item);
 	}
 	(item == NULL) ? NULL : (item = (g_object_unref (item), NULL));
@@ -448,7 +461,7 @@ void dvb_scanner_start_scan (DVBScanner* self) {
 	_tmp4 = NULL;
 	_tmp3 = NULL;
 	_tmp2 = NULL;
-	g_debug ("Scanner.vala:217: Received all tables: %s (pat: %s, sdt: %s, nit: %s)", _tmp2 = bool_to_string (all_tables), _tmp3 = bool_to_string (self->priv->pat_arrived), _tmp4 = bool_to_string (self->priv->sdt_arrived), _tmp5 = bool_to_string (self->priv->nit_arrived));
+	g_debug ("Scanner.vala:235: Received all tables: %s (pat: %s, sdt: %s, nit: %s)", _tmp2 = bool_to_string (all_tables), _tmp3 = bool_to_string (self->priv->pat_arrived), _tmp4 = bool_to_string (self->priv->sdt_arrived), _tmp5 = bool_to_string (self->priv->nit_arrived));
 	_tmp5 = (g_free (_tmp5), NULL);
 	_tmp4 = (g_free (_tmp4), NULL);
 	_tmp3 = (g_free (_tmp3), NULL);
@@ -464,10 +477,10 @@ void dvb_scanner_start_scan (DVBScanner* self) {
 		g_signal_emit_by_name (self, "frequency-scanned", old_freq, self->frequencies->length);
 	}
 	if (g_queue_is_empty (self->frequencies)) {
-		g_message ("Scanner.vala:233: Finished scanning");
+		g_message ("Scanner.vala:251: Finished scanning");
 		/* We don't have all the information for those channels
 		 remove them*/
-		g_debug ("Scanner.vala:236: %u channels still have missing or invalid information", (guint) gee_collection_get_size ((GeeCollection*) self->priv->new_channels));
+		g_debug ("Scanner.vala:254: %u channels still have missing or invalid information", (guint) gee_collection_get_size ((GeeCollection*) self->priv->new_channels));
 		{
 			GeeIterator* _sid_it;
 			_sid_it = gee_iterable_iterator ((GeeIterable*) self->priv->new_channels);
@@ -487,7 +500,7 @@ void dvb_scanner_start_scan (DVBScanner* self) {
 	/* Remember that we already scanned this frequency*/
 	freq = 0U;
 	gst_structure_get_uint (self->current_tuning_params, "frequency", &freq);
-	g_debug ("Scanner.vala:252: Starting scanning frequency %u (%u left)", freq, g_queue_get_length (self->frequencies));
+	g_debug ("Scanner.vala:270: Starting scanning frequency %u (%u left)", freq, g_queue_get_length (self->frequencies));
 	gst_element_set_state (self->pipeline, GST_STATE_READY);
 	dvb_scanner_prepare (self);
 	/* Reset PIDs*/
@@ -587,7 +600,7 @@ void dvb_scanner_on_dvb_frontend_stats_structure (DVBScanner* self, const GstStr
 		_tmp0 = FALSE;
 	}
 	if (_tmp0) {
-		g_debug ("Scanner.vala:316: Got lock");
+		g_debug ("Scanner.vala:334: Got lock");
 		dvb_scanner_remove_check_for_lock_timeout (self);
 		self->priv->wait_for_tables_event_id = g_timeout_add_seconds ((guint) 10, _dvb_scanner_wait_for_tables_gsource_func, self);
 	}
@@ -596,7 +609,7 @@ void dvb_scanner_on_dvb_frontend_stats_structure (DVBScanner* self, const GstStr
 
 void dvb_scanner_on_dvb_read_failure_structure (DVBScanner* self) {
 	g_return_if_fail (self != NULL);
-	g_warning ("Scanner.vala:324: Read failure");
+	g_warning ("Scanner.vala:342: Read failure");
 }
 
 
@@ -614,7 +627,7 @@ void dvb_scanner_on_pat_structure (DVBScanner* self, const GstStructure* structu
 	GstElement* dvbsrc;
 	g_return_if_fail (self != NULL);
 	g_return_if_fail (structure != NULL);
-	g_debug ("Scanner.vala:331: Received PAT");
+	g_debug ("Scanner.vala:349: Received PAT");
 	pid_set = (GeeSet*) gee_hash_set_new (G_TYPE_UINT, NULL, NULL, g_direct_hash, g_direct_equal);
 	/* add BASE_PIDS*/
 	gee_collection_add ((GeeCollection*) pid_set, GUINT_TO_POINTER ((guint) 16));
@@ -654,7 +667,7 @@ void dvb_scanner_on_pat_structure (DVBScanner* self, const GstStructure* structu
 		(_pid_it == NULL) ? NULL : (_pid_it = (g_object_unref (_pid_it), NULL));
 	}
 	pids = string_substring (new_pids->str, (glong) 0, new_pids->len - 1);
-	g_debug ("Scanner.vala:363: Setting %d pids: %s", gee_collection_get_size ((GeeCollection*) pid_set), pids);
+	g_debug ("Scanner.vala:381: Setting %d pids: %s", gee_collection_get_size ((GeeCollection*) pid_set), pids);
 	/* We want to parse the pmt as well*/
 	dvbsrc = gst_bin_get_by_name (GST_BIN (self->pipeline), "dvbsrc");
 	g_object_set ((GObject*) dvbsrc, "pids", pids, NULL);
@@ -674,7 +687,7 @@ void dvb_scanner_on_sdt_structure (DVBScanner* self, const GstStructure* structu
 	const GstStructure* service;
 	g_return_if_fail (self != NULL);
 	g_return_if_fail (structure != NULL);
-	g_debug ("Scanner.vala:372: Received SDT");
+	g_debug ("Scanner.vala:390: Received SDT");
 	tsid = 0U;
 	gst_structure_get_uint (structure, "transport-stream-id", &tsid);
 	services = *gst_structure_get_value (structure, "services");
@@ -758,7 +771,7 @@ void dvb_scanner_on_nit_structure (DVBScanner* self, const GstStructure* structu
 	const GstStructure* transport;
 	g_return_if_fail (self != NULL);
 	g_return_if_fail (structure != NULL);
-	g_debug ("Scanner.vala:426: Received NIT");
+	g_debug ("Scanner.vala:444: Received NIT");
 	name = NULL;
 	if (gst_structure_has_field (structure, "network-name")) {
 		char* _tmp1;
@@ -774,7 +787,7 @@ void dvb_scanner_on_nit_structure (DVBScanner* self, const GstStructure* structu
 		_tmp2 = NULL;
 		name = (_tmp2 = g_strdup_printf ("%u", nid), name = (g_free (name), NULL), _tmp2);
 	}
-	g_debug ("Scanner.vala:436: Network name '%s'", name);
+	g_debug ("Scanner.vala:454: Network name '%s'", name);
 	transports = *gst_structure_get_value (structure, "transports");
 	size = gst_value_list_get_size (&transports);
 	transport = NULL;
@@ -795,7 +808,7 @@ void dvb_scanner_on_nit_structure (DVBScanner* self, const GstStructure* structu
 				const GstStructure* _tmp3;
 				delivery_val = *gst_structure_get_value (transport, "delivery");
 				delivery = gst_value_get_structure (&delivery_val);
-				g_debug ("Scanner.vala:455: Received TS %u", tsid);
+				g_debug ("Scanner.vala:473: Received TS %u", tsid);
 				gee_map_set ((GeeMap*) self->transport_streams, GUINT_TO_POINTER (tsid), delivery);
 				freq = 0U;
 				gst_structure_get_uint (delivery, "frequency", &freq);
@@ -855,7 +868,7 @@ void dvb_scanner_on_pmt_structure (DVBScanner* self, const GstStructure* structu
 	const GstStructure* stream;
 	g_return_if_fail (self != NULL);
 	g_return_if_fail (structure != NULL);
-	g_debug ("Scanner.vala:501: Received PMT");
+	g_debug ("Scanner.vala:519: Received PMT");
 	program_number = 0U;
 	gst_structure_get_uint (structure, "program-number", &program_number);
 	if (!dvb_channel_list_contains (self->channels, program_number)) {
@@ -884,7 +897,7 @@ void dvb_scanner_on_pmt_structure (DVBScanner* self, const GstStructure* structu
 				case 0x02:
 				case 0x1b:
 				{
-					g_debug ("Scanner.vala:533: Found video PID %u for channel %u", pid, program_number);
+					g_debug ("Scanner.vala:551: Found video PID %u for channel %u", pid, program_number);
 					dvb_channel_set_VideoPID (dvb_channel, pid);
 					break;
 				}
@@ -893,13 +906,13 @@ void dvb_scanner_on_pmt_structure (DVBScanner* self, const GstStructure* structu
 				case 0x0f:
 				case 0x11:
 				{
-					g_debug ("Scanner.vala:541: Found audio PID %u for channel %u", pid, program_number);
+					g_debug ("Scanner.vala:559: Found audio PID %u for channel %u", pid, program_number);
 					gee_collection_add ((GeeCollection*) dvb_channel_get_AudioPIDs (dvb_channel), GUINT_TO_POINTER (pid));
 					break;
 				}
 				default:
 				{
-					g_debug ("Scanner.vala:546: Other stream type: 0x%02x", stream_type);
+					g_debug ("Scanner.vala:564: Other stream type: 0x%02x", stream_type);
 					break;
 				}
 			}
@@ -911,7 +924,8 @@ void dvb_scanner_on_pmt_structure (DVBScanner* self, const GstStructure* structu
 
 void dvb_scanner_bus_watch_func (DVBScanner* self, GstBus* bus, GstMessage* message) {
 	gboolean _tmp4;
-	gboolean _tmp5;
+	gboolean _tmp10;
+	gboolean _tmp11;
 	g_return_if_fail (self != NULL);
 	g_return_if_fail (bus != NULL);
 	g_return_if_fail (message != NULL);
@@ -962,7 +976,7 @@ void dvb_scanner_bus_watch_func (DVBScanner* self, GstBus* bus, GstMessage* mess
 				_tmp0 = NULL;
 				(gst_message_parse_error (message, &_tmp0, &_tmp2), gerror = (_tmp1 = _tmp0, (gerror == NULL) ? NULL : (gerror = (g_error_free (gerror), NULL)), _tmp1));
 				debug = (_tmp3 = _tmp2, debug = (g_free (debug), NULL), _tmp3);
-				g_critical ("Scanner.vala:573: %s %s", gerror->message, debug);
+				g_critical ("Scanner.vala:591: %s %s", gerror->message, debug);
 				dvb_scanner_Destroy (self);
 				(gerror == NULL) ? NULL : (gerror = (g_error_free (gerror), NULL));
 				debug = (g_free (debug), NULL);
@@ -971,21 +985,13 @@ void dvb_scanner_bus_watch_func (DVBScanner* self, GstBus* bus, GstMessage* mess
 		}
 	}
 	_tmp4 = FALSE;
-	_tmp5 = FALSE;
-	if (self->priv->sdt_arrived) {
-		_tmp5 = self->priv->nit_arrived;
-	} else {
-		_tmp5 = FALSE;
-	}
-	if (_tmp5) {
-		_tmp4 = self->priv->pat_arrived;
+	if (self->priv->nit_arrived) {
+		_tmp4 = self->priv->sdt_arrived;
 	} else {
 		_tmp4 = FALSE;
 	}
+	/* NIT gives us the transport stream, SDT links SID and TS ID */
 	if (_tmp4) {
-		GeeArrayList* del_channels;
-		dvb_scanner_remove_wait_for_tables_timeout (self);
-		del_channels = gee_array_list_new (G_TYPE_UINT, NULL, NULL, g_direct_equal);
 		{
 			GeeIterator* _sid_it;
 			_sid_it = gee_iterable_iterator ((GeeIterable*) self->priv->new_channels);
@@ -998,62 +1004,99 @@ void dvb_scanner_bus_watch_func (DVBScanner* self, GstBus* bus, GstMessage* mess
 				tsid = dvb_channel_get_TransportStreamId (channel);
 				/* Check if already came across the transport stream*/
 				if (gee_map_contains ((GeeMap*) self->transport_streams, GUINT_TO_POINTER (tsid))) {
-					GstStructure* _tmp6;
+					GstStructure* _tmp5;
 					/* add values from Gst.Structure to Channel*/
-					_tmp6 = NULL;
-					dvb_scanner_add_values_from_structure_to_channel (self, _tmp6 = (GstStructure*) gee_map_get ((GeeMap*) self->transport_streams, GUINT_TO_POINTER (tsid)), channel);
-					(_tmp6 == NULL) ? NULL : (_tmp6 = (gst_structure_free (_tmp6), NULL));
-					/* If this fails we may miss video or audio pid,
-					 because we didn't came across the sdt, yet   */
-					if (dvb_channel_is_valid (channel)) {
-						const char* _tmp7;
-						const char* _tmp8;
-						char* type;
-						char* _tmp9;
-						_tmp7 = NULL;
-						if (dvb_channel_get_VideoPID (channel) == 0) {
-							_tmp7 = "Radio";
-						} else {
-							_tmp7 = "TV";
-						}
-						_tmp8 = NULL;
-						type = (_tmp8 = _tmp7, (_tmp8 == NULL) ? NULL : g_strdup (_tmp8));
-						_tmp9 = NULL;
-						g_debug ("Scanner.vala:598: Channel added: %s", _tmp9 = dvb_channel_to_string (channel));
-						_tmp9 = (g_free (_tmp9), NULL);
-						g_signal_emit_by_name (self, "channel-added", dvb_channel_get_Frequency (channel), sid, dvb_channel_get_Name (channel), dvb_channel_get_Network (channel), type, dvb_channel_get_Scrambled (channel));
-						/* Mark channel for deletion of this.new_channels*/
-						gee_collection_add ((GeeCollection*) del_channels, GUINT_TO_POINTER (sid));
-						type = (g_free (type), NULL);
-					} else {
-						char* _tmp10;
-						_tmp10 = NULL;
-						g_debug ("Scanner.vala:605: Channel %u is not valid: %s", sid, _tmp10 = dvb_channel_to_string (channel));
-						_tmp10 = (g_free (_tmp10), NULL);
-					}
+					_tmp5 = NULL;
+					dvb_scanner_add_values_from_structure_to_channel (self, _tmp5 = (GstStructure*) gee_map_get ((GeeMap*) self->transport_streams, GUINT_TO_POINTER (tsid)), channel);
+					(_tmp5 == NULL) ? NULL : (_tmp5 = (gst_structure_free (_tmp5), NULL));
 				} else {
-					g_debug ("Scanner.vala:609: TS %u for channel %u does not exist", tsid, sid);
+					g_debug ("Scanner.vala:610: TS %u for channel %u does not exist", tsid, sid);
+					/* We haven't seen the transport stream, yet
+					 Maybe it comes with a later bus message*/
+					self->priv->nit_arrived = FALSE;
 				}
 				(channel == NULL) ? NULL : (channel = (g_object_unref (channel), NULL));
 			}
 			(_sid_it == NULL) ? NULL : (_sid_it = (g_object_unref (_sid_it), NULL));
 		}
-		/* Only remove those channels we have all the information for*/
-		{
-			GeeIterator* _sid_it;
-			/* Only remove those channels we have all the information for*/
-			_sid_it = gee_iterable_iterator ((GeeIterable*) del_channels);
-			/* Only remove those channels we have all the information for*/
-			while (gee_iterator_next (_sid_it)) {
-				guint sid;
-				/* Only remove those channels we have all the information for*/
-				sid = GPOINTER_TO_UINT (gee_iterator_get (_sid_it));
-				gee_collection_remove ((GeeCollection*) self->priv->new_channels, GUINT_TO_POINTER (sid));
+		/* We received all tables at least once. Add valid channels.*/
+		if (self->priv->pat_arrived) {
+			GeeArrayList* del_channels;
+			del_channels = gee_array_list_new (G_TYPE_UINT, NULL, NULL, g_direct_equal);
+			{
+				GeeIterator* _sid_it;
+				_sid_it = gee_iterable_iterator ((GeeIterable*) self->priv->new_channels);
+				while (gee_iterator_next (_sid_it)) {
+					guint sid;
+					DVBChannel* channel;
+					sid = GPOINTER_TO_UINT (gee_iterator_get (_sid_it));
+					channel = dvb_channel_list_get_channel (self->channels, sid);
+					/* If this fails we may miss video or audio pid,
+					 because we didn't came across the sdt, yet   */
+					if (dvb_channel_is_valid (channel)) {
+						const char* _tmp6;
+						const char* _tmp7;
+						char* type;
+						char* _tmp8;
+						_tmp6 = NULL;
+						if (dvb_channel_get_VideoPID (channel) == 0) {
+							_tmp6 = "Radio";
+						} else {
+							_tmp6 = "TV";
+						}
+						_tmp7 = NULL;
+						type = (_tmp7 = _tmp6, (_tmp7 == NULL) ? NULL : g_strdup (_tmp7));
+						_tmp8 = NULL;
+						g_debug ("Scanner.vala:628: Channel added: %s", _tmp8 = dvb_channel_to_string (channel));
+						_tmp8 = (g_free (_tmp8), NULL);
+						g_signal_emit_by_name (self, "channel-added", dvb_channel_get_Frequency (channel), sid, dvb_channel_get_Name (channel), dvb_channel_get_Network (channel), type, dvb_channel_get_Scrambled (channel));
+						/* Mark channel for deletion of this.new_channels*/
+						gee_collection_add ((GeeCollection*) del_channels, GUINT_TO_POINTER (sid));
+						type = (g_free (type), NULL);
+					} else {
+						char* _tmp9;
+						_tmp9 = NULL;
+						g_debug ("Scanner.vala:635: Channel %u is not valid: %s", sid, _tmp9 = dvb_channel_to_string (channel));
+						_tmp9 = (g_free (_tmp9), NULL);
+					}
+					(channel == NULL) ? NULL : (channel = (g_object_unref (channel), NULL));
+				}
+				(_sid_it == NULL) ? NULL : (_sid_it = (g_object_unref (_sid_it), NULL));
 			}
-			(_sid_it == NULL) ? NULL : (_sid_it = (g_object_unref (_sid_it), NULL));
+			/* Only remove those channels we have all the information for*/
+			{
+				GeeIterator* _sid_it;
+				/* Only remove those channels we have all the information for*/
+				_sid_it = gee_iterable_iterator ((GeeIterable*) del_channels);
+				/* Only remove those channels we have all the information for*/
+				while (gee_iterator_next (_sid_it)) {
+					guint sid;
+					/* Only remove those channels we have all the information for*/
+					sid = GPOINTER_TO_UINT (gee_iterator_get (_sid_it));
+					gee_collection_remove ((GeeCollection*) self->priv->new_channels, GUINT_TO_POINTER (sid));
+				}
+				(_sid_it == NULL) ? NULL : (_sid_it = (g_object_unref (_sid_it), NULL));
+			}
+			(del_channels == NULL) ? NULL : (del_channels = (g_object_unref (del_channels), NULL));
 		}
+	}
+	_tmp10 = FALSE;
+	_tmp11 = FALSE;
+	if (self->priv->sdt_arrived) {
+		_tmp11 = self->priv->nit_arrived;
+	} else {
+		_tmp11 = FALSE;
+	}
+	if (_tmp11) {
+		_tmp10 = self->priv->pat_arrived;
+	} else {
+		_tmp10 = FALSE;
+	}
+	/* If we collect all information we can continue scanning
+	 the next frequency*/
+	if (_tmp10) {
+		dvb_scanner_remove_wait_for_tables_timeout (self);
 		dvb_scanner_start_scan (self);
-		(del_channels == NULL) ? NULL : (del_channels = (g_object_unref (del_channels), NULL));
 	}
 }
 
@@ -1061,7 +1104,7 @@ void dvb_scanner_bus_watch_func (DVBScanner* self, GstBus* bus, GstMessage* mess
 void dvb_scanner_add_new_channel (DVBScanner* self, guint sid) {
 	DVBChannel* new_channel;
 	g_return_if_fail (self != NULL);
-	g_debug ("Scanner.vala:624: Adding new channel with SID %u", sid);
+	g_debug ("Scanner.vala:657: Adding new channel with SID %u", sid);
 	new_channel = dvb_scanner_get_new_channel (self);
 	dvb_channel_set_Sid (new_channel, sid);
 	dvb_channel_list_add (self->channels, new_channel);
