@@ -24,6 +24,8 @@
 #include <float.h>
 #include <math.h>
 #include <stdio.h>
+#include <gee/arraylist.h>
+#include <gee/collection.h>
 #include <gio/gio.h>
 #include "src/Utils.h"
 
