@@ -1,3 +1,21 @@
+/*
+ * Copyright (C) 2008,2009 Sebastian Pölsterl
+ *
+ * This file is part of GNOME DVB Daemon.
+ *
+ * GNOME DVB Daemon is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
+ *
+ * GNOME DVB Daemon is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with GNOME DVB Daemon.  If not, see <http://www.gnu.org/licenses/>.
+ */
 
 #ifndef __SRC_RTSP_MEDIAFACTORY_H__
 #define __SRC_RTSP_MEDIAFACTORY_H__
@@ -7,6 +25,8 @@
 #include <gst/rtsp-server/rtsp-media-factory.h>
 #include <gst/gst.h>
 #include <gst/rtsp/gstrtspconnection.h>
+#include <stdlib.h>
+#include <string.h>
 
 G_BEGIN_DECLS
 
